#The following script uses data from:
     #Norin, T., H. Malte, and T. D. Clark. 2016.
     #Differential plasticity of metabolic rate phenotypes in a tropical fish facing environmental change.
     #Functional Ecology 30: 369-378.

library(MCMCglmm)

#NOTE: The examples below are not inteded as a tutorial on how to use MCMCglmm.
#NOTE: For information on MCMCglmm, see Hadfield, J.D. (2010) Journal of Statistical Software 33 2 1-22.
#NOTE: We have simplified the coding by using MCMCglmm default settings (for priors, nitt, thin, burnin, etc).
#NOTE: Modifying and using this code to analyze other data requires a basic understanding of mixed-effects models and MCMCglmm.
#NOTE: The repeatability estimates produced will be similar but not exactly the same as in the published manuscript.

data<-read.table(file="Norin_et_al_2016_FUNCT_ECOL.csv", header=T, sep=",")
data$ID<-factor(data$ID)
data$trial<-factor(data$TRIAL)
data$OXY<-factor(data$OXY)
data$TEMP<-factor(data$TEMP)
data$SALINITY<-factor(data$SALINITY)
summary(data)

################################################## univariate mixed-effects models in MCMCglmm (random intercepts) ###########################
################################################## univariate mixed-effects models in MCMCglmm (random intercepts) ###########################
################################################## univariate mixed-effects models in MCMCglmm (random intercepts) ###########################
################################################## univariate mixed-effects models in MCMCglmm (random intercepts) ###########################
################################################## univariate mixed-effects models in MCMCglmm (random intercepts) ###########################

#NOTE: this tutorial is not intended to teach the reader how to use MCMCglmm.
#NOTE: The script is simplified by using MCMCglmm default settings (for priors, nitt, thin, burnin, etc). Nitt = total number of iterations, burnin = burn-in period or number of iterations to drop at the beginning (convergence), thin = number of iterations stored in memory
#NOTE: Readers applying this script to their own data should have a basic understanding of linear mixed-effects models (LMMs) and MCMCglmm.
#NOTE: See Hadfield, J.D. (2010) Journal of Statistical Software 33 2 1-22 for information on MCMCglmm. For additional resources, see: https://cran.r-project.org/web/packages/MCMCglmm/vignettes/CourseNotes.pdf , http://devillemereuil.legtux.org/wp-content/uploads/2012/12/tuto_en.pdf
#NOTE: The script will not return the exact same numbers as published in the paper, but teh values will be fairly close.

#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of raw data (i.e., not adjusted for fixed effects) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#
#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of raw data (i.e., not adjusted for fixed effects) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#
#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of raw data (i.e., not adjusted for fixed effects) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#

#run MCMC models without fixed effects
MAS <-MCMCglmm(MASS~1,random=~ID,data=data)
SMR <-MCMCglmm(SMR ~1,random=~ID,data=data)
MMR <-MCMCglmm(MMR ~1,random=~ID,data=data)
#obtain output (see http://devillemereuil.legtux.org/wp-content/uploads/2012/12/tuto_en.pdf for an explanation of the output from MCMCglmm())
summary(MAS)
summary(SMR)
summary(MMR)
#plot the trace of the chain (i.e. evolution of sampled values along the iterations) and the estimation of the posterior density function for the intercept (mean, u), Vind and Ve
#used to check convergence (we want absence of trend in the trace) and ensure that autocorrelation is weak (parameter values are widely spread)
#ID = Vind and units = Ve
plot(MAS)
plot(SMR)
plot(MMR)

#calculate agreement repeatability using the posterior distributions [R = Vind / (Vind + Ve)]
#$VCV is the output of the model corresponding to random effect variances
R.MAS<-MAS$VCV[,"ID"]/(MAS$VCV[,"ID"]+MAS$VCV[,"units"])
R.SMR<-SMR$VCV[,"ID"]/(SMR$VCV[,"ID"]+SMR$VCV[,"units"])
R.MMR<-MMR$VCV[,"ID"]/(MMR$VCV[,"ID"]+MMR$VCV[,"units"])
plot(R.MAS)
plot(R.SMR)
plot(R.MMR)
#get estimates of the marginal parameter modes (i.e. repeatability estimate) and create Highest Posterior Density (HPD) intervals for the parameter estimates (i.e. 95% CI interval)
posterior.mode(R.MAS)
   HPDinterval(R.MAS)
posterior.mode(R.SMR)
   HPDinterval(R.SMR)
posterior.mode(R.MMR)
   HPDinterval(R.MMR)

#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of data adjusted for fixed effects (adjusted repeatability) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#
#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of data adjusted for fixed effects (adjusted repeatability) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#
#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/ analysis of data adjusted for fixed effects (adjusted repeatability) #-*/#-*/#-*/#-*/#-*/#-*/#-*/#-*/#

#run MCMC model with fixed effects to account for treatment effects
adj.MAS <-MCMCglmm(MASS~TRIAL+TEMP+SALINITY+OXY,random=~ID,data=data)
#run MCMC models with fixed effects to account for treatment effects and fit a separate slope for body mass at each trial
adj.SMR <-MCMCglmm(SMR ~TRIAL+TEMP+SALINITY+OXY+at.level(trial,"1"):MASS+at.level(trial,"2"):MASS+at.level(trial,"3"):MASS+at.level(trial,"4"):MASS+at.level(trial,"5"):MASS,random=~ID,data=data)
adj.MMR <-MCMCglmm(MMR ~TRIAL+TEMP+SALINITY+OXY+at.level(trial,"1"):MASS+at.level(trial,"2"):MASS+at.level(trial,"3"):MASS+at.level(trial,"4"):MASS+at.level(trial,"5"):MASS,random=~ID,data=data)
summary(adj.MAS)
summary(adj.SMR)
summary(adj.MMR)
plot(adj.MAS)
plot(adj.SMR)
plot(adj.MMR)

#calculate adjusted repeatability using the posterior distribution
R.adj.MAS<-adj.MAS$VCV[,"ID"]/(adj.MAS$VCV[,"ID"]+adj.MAS$VCV[,"units"])
R.adj.SMR<-adj.SMR$VCV[,"ID"]/(adj.SMR$VCV[,"ID"]+adj.SMR$VCV[,"units"])
R.adj.MMR<-adj.MMR$VCV[,"ID"]/(adj.MMR$VCV[,"ID"]+adj.MMR$VCV[,"units"])
plot(R.adj.MAS)
plot(R.adj.SMR)
plot(R.adj.MMR)
posterior.mode(R.adj.MAS)
   HPDinterval(R.adj.MAS)
posterior.mode(R.adj.SMR)
   HPDinterval(R.adj.SMR)
posterior.mode(R.adj.MMR)
   HPDinterval(R.adj.MMR)

#put everything in a table
R.estimates<-rbind(
cbind(posterior.mode(R.MAS),HPDinterval(R.MAS),posterior.mode(R.adj.MAS),HPDinterval(R.adj.MAS)),
cbind(posterior.mode(R.SMR),HPDinterval(R.SMR),posterior.mode(R.adj.SMR),HPDinterval(R.adj.SMR)),
cbind(posterior.mode(R.MMR),HPDinterval(R.MMR),posterior.mode(R.adj.MMR),HPDinterval(R.adj.MMR)))
row.names(R.estimates)<-c("Body mass","SMR","MMR")
colnames(R.estimates)<-c("R.agreement","low95","high95","R.adjusted","low95","high95")
R.estimates
#ideally this table would also include variance components, which can be extracted as follows:
#Vind and Ve for body mass from the adjusted analysis
posterior.mode(adj.MAS$VCV[,"ID"])    #Vind for body mass from the adjusted analysis
posterior.mode(adj.MAS$VCV[,"units"]) #Ve for body mass from the adjusted analysis


##################################################  make graphs to show variance among and within individuals ###########################
##################################################  make graphs to show variance among and within individuals ###########################
##################################################  make graphs to show variance among and within individuals ###########################
##################################################  make graphs to show variance among and within individuals ###########################
##################################################  make graphs to show variance among and within individuals ###########################

#First we want to run simple linear models, ignoring repeated measures for each individual and extract residuals
model.MASS.0<-lm(MASS~TRIAL+TEMP+SALINITY+OXY,data=data)
model.SMR.0 <-lm(SMR~TRIAL+TEMP+SALINITY+OXY+MASS*trial,data=data)
model.MMR.0 <-lm(MMR~TRIAL+TEMP+SALINITY+OXY+MASS*trial,data=data)
data$resMASS<-model.MASS.0$resid
data$resSMR<-model.SMR.0$resid
data$resMMR<-model.MMR.0$resid

dev.new(width=11,height=8)
par(mfrow=c(3,4),mar=c(2.8,2,1,3),oma=c(2,2,3,0),las=1)

############################# raw BODY MASS #############################

#reaction norm plot
plot(NA,ylim=range(data$MASS),xlim=range(data$TRIAL),bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data, ID==i)
    points(MASS~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(MASS~TRIAL,temp,type="b",pch="")
}
axis(1,at=c(1,2,3,4,5))
mtext("Body mass",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$MASS,by=list(data$ID),mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$MASS)
  plot(MASS~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(MASS~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(MASS~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(MASS~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(MASS~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1, tick = TRUE,las = 2,cex.axis=0.5)
axis(2)
mtext("Body mass",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
legend("topleft",pch=c(1,2,3,4,5),col=c("blue","green","black","red","grey"),c("Trial 1","Trial 2","Trial 3","Trial 4","Trial 5"),bty="n",cex=1,bg="white")
mtext(expression(paste(italic("R"),"=0.77 [0.68-0.83]")),side=1, line=-1.25,adj=1,cex=0.85)

############################# residual BODY MASS #############################

#reaction norm plot
plot(NA,ylim=range(data$resMASS),xlim=range(data$TRIAL), bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data,ID==i)
    points(resMASS~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(resMASS~TRIAL,temp,type="b",pch="")
}
axis(1,at=c(1,2,3,4,5))
mtext("Residual body mass",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$resMASS,by=list(data$ID),mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$resMASS)
  plot(resMASS~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(resMASS~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(resMASS~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(resMASS~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(resMASS~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1,tick=TRUE,las=2,cex.axis=0.5)
axis(2)
mtext("Residual body mass",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
mtext("Whole-animal analysis",side=3,line=0,adj=0.15,cex=2,outer=T)
mtext("Residuals analysis",side=3,line=0,adj=0.85,cex=2,outer=T)
mtext(expression(paste(italic("R"),"=0.96 [0.94-0.97]")),side=1,line=-1.25,adj=1,cex=0.85)

############################# raw SMR #############################

#reaction norm plot
plot(NA,ylim=range(data$SMR),xlim=range(data$TRIAL),bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data,ID==i)
    points(SMR~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(SMR~TRIAL,temp,type="b",pch="")
}
axis(1,at=c(1,2,3,4,5))
mtext("SMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$SMR, by=list(data$ID), mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$SMR)
  plot(SMR~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(SMR~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(SMR~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(SMR~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(SMR~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1,tick=TRUE,las=2,cex.axis=0.5)
axis(2)
mtext("SMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
mtext(expression(paste(italic("R"),"=0.32 [0.22-0.46]")),side=1,line=-1.25,adj=1,cex=0.85)

############################# residual SMR #############################

#reaction norm plot
plot(NA,ylim=range(data$resSMR),xlim=range(data$TRIAL),bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data,ID==i)
    points(resSMR~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(resSMR~TRIAL,temp,type="b",pch="")
}
axis(1,at=c(1,2,3,4,5))
mtext("Residual SMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$resSMR,by=list(data$ID),mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$resSMR)
  plot(resSMR~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(resSMR~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(resSMR~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(resSMR~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(resSMR~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1,tick=TRUE,las=2,cex.axis=0.5)
axis(2)
mtext("Residual SMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
mtext(expression(paste(italic("R"),"=0.39 [0.23-0.51]")),side=1,line=-1.25,adj=1,cex=0.85)

############################# raw MMR #############################

#reaction norm plot
plot(NA,ylim=range(data$MMR),xlim=range(data$TRIAL),bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data,ID==i)
    points(MMR~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(MMR~TRIAL,temp,type="b",pch="")
}
axis(1,at=c(1,2,3,4,5))
mtext("MMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$MMR,by=list(data$ID),mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$MMR)
  plot(MMR~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(MMR~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(MMR~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(MMR~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(MMR~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1,tick=TRUE,las=2,cex.axis=0.5)
axis(2)
mtext("MMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
mtext(expression(paste(italic("R"),"=0.29 [0.17-0.42]")),side=1,line=-1.25,adj=1,cex=0.85)

############################# residual MMR ############################

#reaction norm plot
plot(NA,ylim=range(data$resMMR),xlim=range(data$TRIAL),bty="l",xaxt="n")
for(i in unique(data$ID)){
    temp<-subset(data, ID==i)
    points(resMMR~TRIAL,temp,type="b",pch=temp$TRIAL,col=rep(c("blue","green","black","red","grey"),60))
    points(resMMR~TRIAL,temp,type="b",pch="")
}
axis(1, at=c(1,2,3,4,5))
mtext("Residual MMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Trial",line=2.5,adj=0.5,side=1,cex=1)

#catepillar plot
temp <- aggregate(data$resMMR,by=list(data$ID),mean)
temp <- temp[order(temp$x),]
data$sort.m <- NA
for(i in 1:nrow(temp))data$sort.m[which(data$ID==as.factor(temp[i,1]))] <- i
lmt<-range(data$resMMR)
  plot(resMMR~sort.m,subset(data, TRIAL==1),pch=1, col="blue", xaxt="n",yaxt="n",ylim=lmt,xlab="", bty="l")
points(resMMR~sort.m,subset(data, TRIAL==2),pch=2, col="green",xaxt="n")
points(resMMR~sort.m,subset(data, TRIAL==3),pch=3, col="black",xaxt="n")
points(resMMR~sort.m,subset(data, TRIAL==4),pch=4, col="red",  xaxt="n")
points(resMMR~sort.m,subset(data, TRIAL==5),pch=5, col="grey", xaxt="n")
axis(1,at=1:nrow(temp),labels=temp$Group.1,tick=TRUE,las=2,cex.axis=0.5)
axis(2)
mtext("Residual MMR",line=2.5,adj=0.5,side=2,cex=1,las=3)
mtext("Individuals",line=2.5,adj=0.5,side=1,cex=1)
mtext(expression(paste(italic("R"),"=0.24[0.10-0.35]")),side=1,line=-1.25,adj=1,cex=0.85)


################################################### random-slope models (univariate) ################
################################################### random-slope models (univariate) ################
################################################### random-slope models (univariate) ################
################################################### random-slope models (univariate) ################
################################################### random-slope models (univariate) ################

#allowing individual variation in intercept (mean body mass) and slope (growth)
#following Araya-Ajoy et al. (Methods in Ecology and Evolution 2015, 6, 14621473)
#we standardise the covariate (in this case trial) to a mean of zero and a variance of 2 units of standard deviation:
data$TRIALz<-(data$TRIAL-mean(data$TRIAL))/(2*sd(data$TRIAL))

# we also standardise the response variable (in this case mass) to a mean of zero and a variance of 1:
data$MASSz<-scale(data$MASS)

RS.MAS <-MCMCglmm(MASSz~TRIALz+TEMP+SALINITY+OXY,random=~us(1+TRIALz):ID,data=data)
summary(RS.MAS)

#to see if including random slopes significantly increased the "likelihood" of the model,
#we can compare the DIC with a random intercept model
INT.MAS <-MCMCglmm(MASSz~TRIALz+TEMP+SALINITY+OXY,random=~ID,data=data)
INT.MAS$DIC
RS.MAS$DIC

#we can calculate the correlation between the random intercepts and slopes
#by dividing the covariance by the square root of the product of the variances
R.int.slp<-RS.MAS$VCV[,"TRIALz:(Intercept).ID"]/sqrt((RS.MAS$VCV[,"(Intercept):(Intercept).ID"]*RS.MAS$VCV[,"TRIALz:TRIALz.ID"]))
posterior.mode(R.int.slp)
   HPDinterval(R.int.slp)




################################################## MULTIvariate mixed-effects models in MCMCglmm ###########################
################################################## MULTIvariate mixed-effects models in MCMCglmm ###########################
################################################## MULTIvariate mixed-effects models in MCMCglmm ###########################
################################################## MULTIvariate mixed-effects models in MCMCglmm ###########################
################################################## MULTIvariate mixed-effects models in MCMCglmm ###########################

#For those interested, here is the code to run multivariate random slope models
#The model below will fit a random intercept and a random slope to both SMR and MMR and a covariance between all of these variance components.
#this would be useful, for example, to look at the among and within-individual relationships between SMR and MMR

#first standardise the response variables (in this case SMR and MMR) to a mean of zero and a variance of 1:
data$SMRz<-scale(data$SMR)
data$MMRz<-scale(data$MMR)
#set priors
PRIOR.1<-list(R=list(V=diag(2),nu=1.002),G=list(G1=list(V=diag(4), nu=3.002)))
MODEL.1<-MCMCglmm(cbind(MMRz,SMRz)~trait-1+trait:TRIALz+trait:TEMP+trait:SALINITY+trait:OXY+trait:MASS,
                                      random=~us(trait+trait:TRIALz):ID,
                                      rcov = ~us(trait):units,
                                      family=c("gaussian","gaussian"),
                                      prior=PRIOR.1,data=data)
summary(MODEL.1)

#we can calculate the among-individual correlation at the intercept level as follows:
R.int.SMR.MMR<-MODEL.1$VCV[,"traitMMRz:traitSMRz.ID"]/sqrt((MODEL.1$VCV[,"traitMMRz:traitMMRz.ID"]*MODEL.1$VCV[,"traitSMRz:traitSMRz.ID"]))
posterior.mode(R.int.SMR.MMR)
   HPDinterval(R.int.SMR.MMR)

#we can calculate the among-individual correlation at the slope level as follows:
R.slp.SMR.MMR<-MODEL.1$VCV[,"traitSMRz:TRIALz:traitMMRz:TRIALz.ID"]/sqrt((MODEL.1$VCV[,"traitMMRz:TRIALz:traitMMRz:TRIALz.ID"]*MODEL.1$VCV[,"traitSMRz:TRIALz:traitSMRz:TRIALz.ID"]))
posterior.mode(R.slp.SMR.MMR)
   HPDinterval(R.slp.SMR.MMR)

#we can calculate the residual (within)-individual correlation as follows:
R.e.SMR.MMR<-MODEL.1$VCV[,"traitSMRz:traitMMRz.units"]/sqrt((MODEL.1$VCV[,"traitSMRz:traitSMRz.units"]*MODEL.1$VCV[,"traitSMRz:traitSMRz.units"]))
posterior.mode(R.e.SMR.MMR)
   HPDinterval(R.e.SMR.MMR)
#got it?

#The model below will fit a random intercept and a random slope to each trait (mass, SMR, and MMR)
#and a covariance between all of these variance component.
#this approach is useful, for example, to look at the relationship between SMR and growth
#which is captured by the covariance between the random intercept for SMR and random slope for MASS
#this allows hypothesis testing in a one-step approach instead of first calculating residuals and then applying stats
PRIOR.2<-list(R=list(V=diag(3),nu=2.002),G=list(G1=list(V=diag(6), nu=5.002)))
MODEL.2<-MCMCglmm(cbind(MASSz,SMRz,MMRz)~trait-1+trait:TRIALz+trait:TEMP+trait:SALINITY+trait:OXY,
                                      random=~us(trait+trait:TRIALz):ID,
                                      rcov = ~us(trait):units,
                                      family=c("gaussian","gaussian","gaussian"),
                                      prior=PRIOR.2,data=data)
summary(MODEL.2)

#here is the correlation between growth and SMR:
R.growth.SMR<-MODEL.2$VCV[,"traitMASSz:TRIALz:traitSMRz.ID"]/sqrt((MODEL.2$VCV[,"traitMASSz:TRIALz:traitMASSz:TRIALz.ID"]*MODEL.2$VCV[,"traitSMRz:traitSMRz.ID"]))
posterior.mode(R.growth.SMR)
   HPDinterval(R.growth.SMR)

#is there also a residual correlation between SMR and MMR in this model:
R.e.SMR.MMR<-MODEL.2$VCV[,"traitSMRz:traitMMRz.units"]/sqrt((MODEL.2$VCV[,"traitSMRz:traitSMRz.units"]*MODEL.2$VCV[,"traitSMRz:traitSMRz.units"]))
posterior.mode(R.e.SMR.MMR)
   HPDinterval(R.e.SMR.MMR)



##################################################  bivariate mixed-effects model in MCMCglmm ###########################
##################################################  bivariate mixed-effects model in MCMCglmm ###########################
##################################################  bivariate mixed-effects model in MCMCglmm ###########################
##################################################  bivariate mixed-effects model in MCMCglmm ###########################
##################################################  bivariate mixed-effects model in MCMCglmm ###########################

#The code below runs a bivariate mixed model to estimate the among- and within-individual correlations between SMR and MMR
#Here we had to specify priors, and set longer nitt, thin, burnin.
NITT=13000*3; THIN=10*3;BURNIN=3000*3
PRIORS<-list(R = list(V = diag(2), nu = 1.02),
             G = list(G1= list(V=diag(2), nu = 1.02)))

data$SMRz<-scale(data$SMR)
data$MMRz<-scale(data$MMR)

#run the bivariate moxed model
BIVAR<-MCMCglmm(cbind(SMRz,MMRz)~trait-1+trait:TRIAL+trait:TEMP+trait:SALINITY+trait:OXY+
                       trait:at.level(trial, "1"):MASS+trait:at.level(trial, "2"):MASS+trait:at.level(trial, "3"):MASS+trait:at.level(trial, "4"):MASS+trait:at.level(trial, "5"):MASS,
            random=~us(trait):ID,
            rcov = ~us(trait):units,
            family=c("gaussian","gaussian"),
            data=data,nitt=NITT,thin=THIN,burnin=BURNIN, prior=PRIORS)
summary(BIVAR)
plot(BIVAR)

#using the posterior distribution, we calculate the correlations at the phenotypic, among-individual, and within-individual levels as follows:
#among-individual covariance divided by the product of the among-individual variances
among.cor <-BIVAR$VCV[,"traitMMRz:traitSMRz.ID"]   /sqrt(BIVAR$VCV[,"traitSMRz:traitSMRz.ID"]   *BIVAR$VCV[,"traitMMRz:traitMMRz.ID"])
#within-individual covariance divided by the product of the within-individual variances
within.cor<-BIVAR$VCV[,"traitMMRz:traitSMRz.units"]/sqrt(BIVAR$VCV[,"traitSMRz:traitSMRz.units"]*BIVAR$VCV[,"traitMMRz:traitMMRz.units"])
#phenotypic covariance (sum of the among- and within-individual covariances) divided by the product of the phenotypic variances (sum of the among- and within-individual variances)
pheno.cor <-BIVAR$VCV[,"traitMMRz:traitSMRz.ID"]+BIVAR$VCV[,"traitMMRz:traitSMRz.units"]   /
            sqrt((BIVAR$VCV[,"traitSMRz:traitSMRz.ID"]+BIVAR$VCV[,"traitSMRz:traitSMRz.units"])*(BIVAR$VCV[,"traitMMRz:traitMMRz.ID"]+BIVAR$VCV[,"traitMMRz:traitMMRz.units"]))
posterior.mode(among.cor)
   HPDinterval(among.cor)
posterior.mode(within.cor)
   HPDinterval(within.cor)
posterior.mode(pheno.cor)
   HPDinterval(pheno.cor)
plot(among.cor)
plot(within.cor)
plot(pheno.cor)

#Seems like the residual correlation obtained in the multivariate random slope models above were due to changes in scaling relationships at different trials
#try running MODEL.1 again, but this time specifying a different mass effect for each trial:
PRIOR.1<-list(R=list(V=diag(2),nu=1.002),G=list(G1=list(V=diag(4), nu=3.002)))
MODEL.1<-MCMCglmm(cbind(MMRz,SMRz)~trait-1+trait:TRIALz+trait:TEMP+trait:SALINITY+trait:OXY+
                       trait:at.level(trial, "1"):MASS+trait:at.level(trial, "2"):MASS+trait:at.level(trial, "3"):MASS+trait:at.level(trial, "4"):MASS+trait:at.level(trial, "5"):MASS,
                                      random=~us(trait+trait:TRIALz):ID,
                                      rcov = ~us(trait):units,
                                      family=c("gaussian","gaussian"),
                                      prior=PRIOR.1,data=data)
summary(MODEL.1)

#then re-calculate the residual (within)-individual correlation as follows:
R.e.SMR.MMR<-MODEL.1$VCV[,"traitSMRz:traitMMRz.units"]/sqrt((MODEL.1$VCV[,"traitSMRz:traitSMRz.units"]*MODEL.1$VCV[,"traitSMRz:traitSMRz.units"]))
posterior.mode(R.e.SMR.MMR)
   HPDinterval(R.e.SMR.MMR)
#not significant anymore... this shows how careful we need to be when interpreting residual correlations
#See Brommer, J. E. 2013. On between-individual and residual (co)variances in the study of animal personality: are you willing to take the individual gambit? Behavioral Ecology and Sociobiology 67:1027-1032.
